@@ -13,7 +13,7 @@ from data import *
 import arch
 from config import conf
 from packet import Gen
-from utils import warning,get_temp_file,PcapReader
+from utils import warning,get_temp_file,PcapReader,wrpcap
 import plist
 from error import log_runtime,log_interactive
 from base_classes import SetGen
@@ -207,9 +207,9 @@ def sndrcv(pks, pkt, timeout = None, inter = 0, verbose=None, chainCC=0, retry=0
     return plist.SndRcvList(ans),plist.PacketList(remain,"Unanswered")
 
 
-def __gen_send(s, x, inter=0, loop=0, count=None, verbose=None, realtime=None, *args, **kargs):
+def __gen_send(s, x, inter=0, loop=0, count=None, verbose=None, realtime=None, return_packets=False, *args, **kargs):
     if type(x) is str:
-        x = Raw(load=x)
+        x = conf.raw_layer(load=x)
     if not isinstance(x, Gen):
         x = SetGen(x)
     if verbose is None:
@@ -218,10 +218,12 @@ def __gen_send(s, x, inter=0, loop=0, count=None, verbose=None, realtime=None, *
     if count is not None:
         loop = -count
     elif not loop:
-        loop=-1
-    dt0 = None
+        loop = -1
+    if return_packets:
+        sent_packets = plist.PacketList()
     try:
         while loop:
+            dt0 = None
             for p in x:
                 if realtime:
                     ct = time.time()
@@ -230,8 +232,10 @@ def __gen_send(s, x, inter=0, loop=0, count=None, verbose=None, realtime=None, *
                         if st > 0:
                             time.sleep(st)
                     else:
-                        dt0 = ct-p.time 
+                        dt0 = ct-p.time
                 s.send(p)
+                if return_packets:
+                    sent_packets.append(p)
                 n += 1
                 if verbose:
                     os.write(1,".")
@@ -243,20 +247,25 @@ def __gen_send(s, x, inter=0, loop=0, count=None, verbose=None, realtime=None, *
     s.close()
     if verbose:
         print "\nSent %i packets." % n
+    if return_packets:
+        return sent_packets
         
 @conf.commands.register
-def send(x, inter=0, loop=0, count=None, verbose=None, realtime=None, *args, **kargs):
+def send(x, inter=0, loop=0, count=None, verbose=None, realtime=None, return_packets=False, *args, **kargs):
     """Send packets at layer 3
 send(packets, [inter=0], [loop=0], [verbose=conf.verb]) -> None"""
-    __gen_send(conf.L3socket(*args, **kargs), x, inter=inter, loop=loop, count=count,verbose=verbose, realtime=realtime)
+    return __gen_send(conf.L3socket(*args, **kargs), x, inter=inter, loop=loop, count=count,verbose=verbose,
+                      realtime=realtime, return_packets=return_packets)
 
 @conf.commands.register
-def sendp(x, inter=0, loop=0, iface=None, iface_hint=None, count=None, verbose=None, realtime=None, *args, **kargs):
+def sendp(x, inter=0, loop=0, iface=None, iface_hint=None, count=None, verbose=None, realtime=None,
+          return_packets=False, *args, **kargs):
     """Send packets at layer 2
 sendp(packets, [inter=0], [loop=0], [verbose=conf.verb]) -> None"""
     if iface is None and iface_hint is not None:
         iface = conf.route.route(iface_hint)[0]
-    __gen_send(conf.L2socket(iface=iface, *args, **kargs), x, inter=inter, loop=loop, count=count, verbose=verbose, realtime=realtime)
+    return __gen_send(conf.L2socket(iface=iface, *args, **kargs), x, inter=inter, loop=loop, count=count,
+                      verbose=verbose, realtime=realtime, return_packets=return_packets)
 
 @conf.commands.register
 def sendpfast(x, pps=None, mbps=None, realtime=None, loop=0, file_cache=False, iface=None):
@@ -273,9 +282,9 @@ def sendpfast(x, pps=None, mbps=None, realtime=None, loop=0, file_cache=False, i
     if pps is not None:
         argv.append("--pps=%i" % pps)
     elif mbps is not None:
-        argv.append("--mbps=%i" % mbps)
+        argv.append("--mbps=%f" % mbps)
     elif realtime is not None:
-        argv.append("--multiplier=%i" % realtime)
+        argv.append("--multiplier=%f" % realtime)
     else:
         argv.append("--topspeed")
 
@@ -566,8 +575,8 @@ stop_filter: python function applied to each packet to determine
     if timeout is not None:
         stoptime = time.time()+timeout
     remain = None
-    while 1:
-        try:
+    try:
+        while 1:
             if timeout is not None:
                 remain = stoptime-time.time()
                 if remain <= 0:
@@ -590,12 +599,76 @@ stop_filter: python function applied to each packet to determine
                     break
                 if count > 0 and c >= count:
                     break
-        except KeyboardInterrupt:
-            break
+    except KeyboardInterrupt:
+        pass
     if opened_socket is None:
         s.close()
     return plist.PacketList(lst,"Sniffed")
 
+
+@conf.commands.register
+def bridge_and_sniff(if1, if2, count=0, store=1, offline=None, prn = None, lfilter=None, L2socket=None, timeout=None,
+                     stop_filter=None, *args, **kargs):
+    """Forward traffic between two interfaces and sniff packets exchanged
+bridge_and_sniff([count=0,] [prn=None,] [store=1,] [offline=None,] [lfilter=None,] + L2Socket args) -> list of packets
+
+  count: number of packets to capture. 0 means infinity
+  store: wether to store sniffed packets or discard them
+    prn: function to apply to each packet. If something is returned,
+         it is displayed. Ex:
+         ex: prn = lambda x: x.summary()
+lfilter: python function applied to each packet to determine
+         if further action may be done
+         ex: lfilter = lambda x: x.haslayer(Padding)
+timeout: stop sniffing after a given time (default: None)
+L2socket: use the provided L2socket
+stop_filter: python function applied to each packet to determine
+             if we have to stop the capture after this packet
+             ex: stop_filter = lambda x: x.haslayer(TCP)
+    """
+    c = 0
+    if L2socket is None:
+        L2socket = conf.L2socket
+    s1 = L2socket(iface=if1)
+    s2 = L2socket(iface=if2)
+    peerof={s1:s2,s2:s1}
+    label={s1:if1, s2:if2}
+    
+    lst = []
+    if timeout is not None:
+        stoptime = time.time()+timeout
+    remain = None
+    try:
+        while True:
+            if timeout is not None:
+                remain = stoptime-time.time()
+                if remain <= 0:
+                    break
+            ins,outs,errs = select([s1,s2],[],[], remain)
+            for s in ins:
+                p = s.recv()
+                if p is not None:
+                    peerof[s].send(p.original)
+                    if lfilter and not lfilter(p):
+                        continue
+                    if store:
+                        p.sniffed_on = label[s]
+                        lst.append(p)
+                    c += 1
+                    if prn:
+                        r = prn(p)
+                        if r is not None:
+                            print "%s: %s" % (label[s],r)
+                    if stop_filter and stop_filter(p):
+                        break
+                    if count > 0 and c >= count:
+                        break
+    except KeyboardInterrupt:
+        pass
+    finally:
+        return plist.PacketList(lst,"Sniffed")
+
+
 @conf.commands.register
 def tshark(*args,**kargs):
     """Sniff packets and print them calling pkt.show(), a bit like text wireshark"""
